from datetime import datetime
from .driver import Driver
from .settings import Settings
from .user import User
import PyInquirer
from .validators import AmountValidator, MonthValidator, LimitValidator, NumberValidator, TimeValidator, DateValidator, DurationValidator, ExpirationValidator, ListValidator

class Discount:

	def __init__(self):
		self.amount = None
		self.months = None
		self.username = None
		self.gotten = False

	def apply(self):
		self.get()
		if not self.gotten: return
		if Settings.is_prompt():
			if not Settings.prompt("Discount"): return
		if str(self.get_username()).lower() == "all":
			users = User.get_all_users()
		elif str(self.get_username()).lower() == "recent":
			users = User.get_new_users()
		elif str(self.get_username()).lower() == "favorite":
			users = User.get_favorite_users()
		else: users = [self]
		successful = False
		for user in users:
			self.username = user.username
			successful_ = Driver.discount_user(self)
			if successful_: successful = successful_
		return successful

	def get(self):
		if self.gotten: return
		gotten = self.get_username()
		if not gotten: return
		gotten = self.get_amount()
		if not gotten: return
		gotten = self.get_months()
		if not gotten: return
		self.gotten = True

	def get_amount(self):
		if self.amount: return self.amount
		amount = Settings.get_amount() or None
		print(amount)
		if amount: return amount
		if not Settings.prompt("amount"): return None
		question = {
			'type': 'input',
			'name': 'amount',
			'message': 'Amount:',
			'validate': AmountValidator,
			'filter': lambda val: int(myround(int(val)))
		}
		answers = PyInquirer.prompt(question)
		amount = answers["amount"]
		if not Settings.confirm(amount): return self.get_amount()
		self.amount = amount
		return self.amount

	def get_months(self):
		if self.months: return self.months
		months = Settings.get_months() or None
		if months: return months
		if not Settings.prompt("months"): return None
		question = {
			'type': 'input',
			'name': 'months',
			'message': 'Months:',
			'validate': MonthValidator,
			'filter': lambda val: int(val)
		}
		answers = PyInquirer.prompt(question)
		months = answers["months"]
		if not Settings.confirm(months): return self.get_months()
		self.months = months
		return self.months

	def get_username(self):
		if self.username: return self.username
		username = User.select_user()
		self.username = username
		return self.username

class Poll:

	def __init__(self):
		self.duration = None
		self.questions = []
		self.gotten = False

	def apply(self):
		self.get()
		if not self.gotten: return
		if not Settings.prompt("Poll"): return
		return True

	def check(self):
		if len(self.get_questions()) > 0: return True
		if self.get_duration(): return True
		return False

	def get(self):
		if self.gotten: return
		gotten = self.get_duration()
		if not gotten: return
		gotten = self.get_questions()
		if not gotten: return
		self.gotten = True

	def get_questions(self):
		if len(self.questions) > 0: return self.questions
		questions = Settings.get_questions() or []
		if len(questions) > 0: return questions
		if not Settings.prompt("questions"): return []
		print("Enter Questions")
		while True:
			question = {
				'type': 'input',
				'name': 'question',
				'message': 'Question:',
			}
			answers = PyInquirer.prompt(question)
			question = answers["question"]
			if str(question) == "": break
			questions.append(question)
		if not Settings.confirm(questions): return self.get_questions()
		self.questions = questions
		return self.questions
	
	def get_duration(self): # months
		if self.duration: return self.duration
		duration = Settings.get_duration() or None
		if duration: return duration
		if not Settings.prompt("duration"): return None
		question = {
			'type': 'input',
			'name': 'duration',
			'message': 'Duration [1, 3, 7, 99 (\'No Limit\')]',
			'validate': DurationValidator
		}
		answers = PyInquirer.prompt(question)
		duration = answers["duration"]
		if not Settings.confirm(duration): return self.get_duration()
		self.duration = duration
		return self.duration

class Promotion:

	def __init__(self):
		self.subscriptionLimit = None
		self.expiration = None
		self.duration = None
		self.user = None
		self.message = None
		self.gotten = False

	# requires the copy/paste and email steps
	def create_trial_link(self):
		self.get()
		if not self.gotten: return
		if not Settings.prompt("Promotion"): return
		# limit, expiration, months, user
		Driver.create_trial_link(self)

	# apply discount directly to user on user's profile page
	def apply_to_user():
		self.get()
		if not self.gotten: return
		if not Settings.prompt("Promotion"): return
		# user, expiration, months, message
		Driver.promotion_user_directly(self)

	def get():
		if self.gotten: return
		gotten = self.get_user()
		if not gotten: return
		gotten = self.get_expiration()
		if not gotten: return
		gotten = self.get_limit()
		if not gotten: return
		gotten = self.get_duration()
		if not gotten: return
		gotten = self.get_message()
		if not gotten: return
		self.gotten = True

	def get_expiration(self):
		if self.expiration: return self.expiration
		expiration = Settings.get_expiration() or None
		if expiration: return expiration
		if not Settings.prompt("expiration"): return None
		question = {
			'type': 'input',
			'name': 'expiration',
			'message': 'Expiration [1, 3, 7, 99 (\'No Limit\')]',
			'validate': ExpirationValidator
		}
		answers = PyInquirer.prompt(question)
		expiration = answers["expiration"]
		if not Settings.confirm(expiration): return self.get_expiration()
		self.expiration = expiration
		return self.expiration

	def get_limit():
		if self.limit: return self.limit
		limit = Settings.get_limit() or None
		if limit: return limit
		if not limit.prompt("limit"): return None
		question = {
			'type': 'input',
			'name': 'limit',
			'message': 'Expiration [1, 3, 7, 99 (\'No Limit\')]',
			'validate': LimitValidator
		}
		answers = PyInquirer.prompt(question)
		limit = answers["limit"]
		if not Settings.confirm(limit): return self.get_limit()
		self.limit = limit
		return self.limit

	def get_message(self):
		if self.message != "": return self.message
		message = Settings.get_text() or None
		if message: return message
		if not Settings.prompt("message"): return ""
		question = {
			'type': 'input',
			'name': 'message',
			'message': 'Message:'
		}
		answers = PyInquirer.prompt(question)
		message = answers["message"]
		if not Settings.confirm(message): return self.get_text()
		self.message = message
		return self.message

	def get_duration(self): # months
		if self.duration: return self.duration
		duration = Settings.get_duration() or None
		if duration: return duration
		if not Settings.prompt("duration"): return None
		question = {
			'type': 'input',
			'name': 'duration',
			'message': 'Duration [1, 3, 7, 99 (\'No Limit\')]',
			'validate': DurationValidator
		}
		answers = PyInquirer.prompt(question)
		duration = answers["duration"]
		if not Settings.confirm(duration): return self.get_duration()
		self.duration = duration
		return self.duration

	def get_user():
		if self.user: return self.user
		user = User.select_user()
		self.user = user
		return self.user

class Schedule:

	def __init__(self):
		self.date = None
		self.time = None
		##
		self.hour = "00"
		self.minute = "00"
		self.year = "0"
		self.month = "0"
		self.day = "0"
		##
		self.gotten = False

	def apply(self):
		self.get()
		if not self.gotten: return
		if not Settings.prompt("Schedule"): return

	def check(self):
		if self.get_date() and self.get_time(): return True
		return False

	def get(self):
		if self.gotten: return
		if self.get_date():
			date = datetime.strptime(str(self.get_date()), "%Y-%m-%d %H:%M:%S")
			self.year = date.year
			self.month = date.strftime("%B")
			self.day = date.day
			if self.get_time():
				self.hour = date.hour
				self.minute = date.minute
		self.gotten = True

	def get_date(self):
		if self.date: return self.date
		date = Settings.get_date() or None
		if date: return date
		schedule = Settings.get_schedule() or None
		if schedule:
			date = datetime.strptime(str(schedule), "%Y-%m-%d %H:%M:%S")
			self.date = date.date()
			return self.date
		if not Settings.prompt("date"): return None
		question = {
			'type': 'input',
			'name': 'date',
			'message': 'Enter a date (MM-DD-YYYY):',
			'validate': DateValidator
		}
		answers = PyInquirer.prompt(question)
		date = answers["date"]
		if not Settings.confirm(date): return self.get_date()
		self.date = date
		return self.date

	def get_time(self):
		if self.time: return self.time
		time = Settings.get_time() or None
		if time: return time
		schedule = Settings.get_schedule() or None
		if schedule:
			time = datetime.strptime(str(schedule), "%Y-%m-%d %H:%M:%S")
			self.time = time.time()
			return self.time
		if not Settings.prompt("time"): return None
		question = {
			'type': 'input',
			'name': 'time',
			'message': 'Enter a time (HH:MM):',
			'validate': TimeValidator
		}
		answers = PyInquirer.prompt(question)
		time = answers["time"]
		if not Settings.confirm(time): return self.get_time()
		self.time = time
		return self.time

# round to 5
def myround(x, base=5):
	return base * round(x/base)


#!/usr/bin/python3
# Profile Settings

# from OnlySnarf.driver import Driver
import PyInquirer

class Profile:

    # profile settings are either:
    #   enabled or disabled
    #   display text, variable type, variable name in settings
    # this is the absolute worst way to do this
    def __init__(self, data):
        if not data: data = Profile.fill_data()
        # url path or file upload
        self.coverImage = data.get("coverImage") or None
        # url path or file upload
        self.profilePhoto = data.get("profilePhoto") or None
        # text
        self.displayName = data.get("displayName") or ""
        # text in $
        # minimum $4.99 or free
        self.subscriptionPrice = data.get("subscriptionPrice") or "4.99"
        # text
        self.about = data.get("about") or ""
        # text
        self.location = data.get("location") or ""
        # text as url
        self.websiteURL = data.get("websiteURL") or None
        # text
        self.username = data.get("username") or ""
        # text, can't be changed
        self.email = data.get("email") or ""
        # text
        self.password = data.get("password") or ""
        # enabled or disabled
        self.emailNotifs = data.get("emailNotifs") or False
        # enabled or disabled
        self.emailNotifsNewReferral = data.get("emailNotifsNewReferral") or False
        # enabled or disabled
        self.emailNotifsNewStream = data.get("emailNotifsNewStream") or False
        # enabled or disabled
        self.emailNotifsNewSubscriber = data.get("emailNotifsNewSubscriber") or False
        # enabled or disabled
        self.emailNotifsNewTip = data.get("emailNotifsNewTip") or False
        # enabled or disabled
        self.emailNotifsRenewal = data.get("emailNotifsRenewal") or False
        # enabled or disabled
        self.emailNotifsNewLikes = data.get("emailNotifsNewLikes") or False
        # enabled or disabled
        self.emailNotifsNewPosts = data.get("emailNotifsNewPosts") or False
        # enabled or disabled
        self.emailNotifsNewPrivMessages = data.get("emailNotifsNewPrivMessages") or False
        # enabled or disabled
        self.siteNotifs = data.get("siteNotifs") or False
        # enabled or disabled
        self.siteNotifsNewComment = data.get("siteNotifsNewComment") or False
        # enabled or disabled
        self.siteNotifsNewFavorite = data.get("siteNotifsNewFavorite") or False
        # enabled or disabled
        self.siteNotifsDiscounts = data.get("siteNotifsDiscounts") or False
        # enabled or disabled
        self.siteNotifsNewSubscriber = data.get("siteNotifsNewSubscriber") or False
        # enabled or disabled
        self.siteNotifsNewTip = data.get("siteNotifsNewTip") or False
        # enabled or disabled
        self.toastNotifs = data.get("toastNotifs") or False
        # enabled or disabled
        self.toastNotifsNewComment = data.get("toastNotifsNewComment") or False
        # enabled or disabled
        self.toastNotifsNewFavorite = data.get("toastNotifsNewFavorite") or False
        # enabled or disabled
        self.toastNotifsNewSubscriber = data.get("toastNotifsNewSubscriber") or False
        # enabled or disabled
        self.toastNotifsNewTip = data.get("toastNotifsNewTip") or False
        # enabled or disabled
        self.fullyPrivate = data.get("fullyPrivate") or False
        # enabled or disabled
        self.enableComments = data.get("enableComments") or False
        # enabled or disabled
        self.showFansCount = data.get("showFansCount") or False
        # enabled or disabled
        self.showPostsTip = data.get("showPostsTip") or False
        # enabled or disabled
        self.publicFriendsList = data.get("publicFriendsList") or False
        # selection of countries
        self.ipCountry = data.get("ipCountry") or Profile.get_country_list()
        # text of ip ranges
        self.ipIP = data.get("ipIP") or ""
        # enabled or disabled
        self.watermark = data.get("watermark") or True
        # enabled or disabled
        self.watermarkPhoto = data.get("watermarkPhoto") or False
        # enabled or disabled
        self.watermarkVideo = data.get("watermarkVideo") or False
        # the custom watermark text
        self.watermarkText = data.get("watermarkText") or ""
        if self.username and str(self.username) != "" and self.watermarkText == "":
            self.watermarkText = "OnlyFans.com/{}".format(self.username)
        # the obs live server
        self.liveServer = data.get("liveServer") or ""
        # the obs live server key
        self.liveServerKey = data.get("liveServerKey") or ""

    def __getitem__(self, key):
        return getattr(self, key)

    def __setitem__(self, key, val):
        return setattr(self, key, val)

    def set(self, key, value):
        self[key] = value

    def get(self):
        pass

    # def set(self):
    #     pass

    def sync_from_profile(self):
        # syncs profile settings w/ onlyfans
        pass

    def sync_to_profile(self):
        # syncs profile settings to onlyfans
        pass

    def sync_to_profile_tab(self, label):
        # syncs profile settings for the specificed tab to onlyfans
        pass

    def syncFrom(self):
        # opens every settings page in the browser from pages or all
        # gets necessary variables from browser
        variables = Profile.get_setting_variables()
        pages = []
        for var in variables:
            if var[1] not in pages:
                pages.append(var[1])
        for page in pages:
            pass
            # Driver.go_to_settings_page(page)
        pass            

    def syncTo(self):
        # opens every settings page in the browser from pages or all
        # updates necessary variables in browser
        pass

    @staticmethod
    def get_country_list():
        return ["USA","Canada"]

    @staticmethod
    def get_pages():
        return ["profile", "advanced", "messaging", "notifications", "security", "story", "other"]

    @staticmethod
    def get_variables_for_page(page):
        variables = get_settings_variables()
        vars_ = []
        for var in variables:
            if str(var[1]) == str(page):
                vars_.append(var)
        return vars_

    @staticmethod
    def get_setting_variable(key):
        variables = get_setting_variables()
        for var in variables:
            if str(var[0]) == str(key):
                return var
        return None

    def update_value(self, key, value):
        try:
            if key == None or str(key) == "" or str(key) == "None": return
            self[key] = value
        except Exception as e:
            print(e)


    # set default values / get from profile online
    def get():
        pass


    def update():
        # called by snarf
        # add the appropriate get conditions to this profile class
        pass














    
    @staticmethod
    def fill_data():
        prof = {
            "coverImage": None,
            "profilePhoto": None,
            "displayName": "",
            "subscriptionPrice": "4.99",
            "about": "",
            "location": "",
            "websiteURL": None,
            "username": "",
            "email": "",
            "password": "",
            "emailNotifs": False,
            "emailNotifsNewReferral": False,
            "emailNotifsNewStream": False,
            "emailNotifsNewSubscriber": False,
            "emailNotifsNewTip": False,
            "emailNotifsRenewal": False,
            "emailNotifsNewLikes": False,
            "emailNotifsNewPosts": False,
            "emailNotifsNewPrivMessages": False,
            "siteNotifs": False,
            "siteNotifsNewComment": False,
            "siteNotifsNewFavorite": False,
            "siteNotifsDiscounts": False,
            "siteNotifsNewSubscriber": False,
            "siteNotifsNewTip": False,
            "toastNotifs": False,
            "toastNotifsNewComment": False,
            "toastNotifsNewFavorite": False,
            "toastNotifsNewSubscriber": False,
            "toastNotifsNewTip": False,
            "fullyPrivate": False,
            "enableComments": False,
            "showFansCount": False,
            "showPostsTip": False,
            "publicFriendsList": False,
            "ipCountry": Profile.get_country_list(),
            "ipIP": "",
            "watermark": True,
            "watermarkPhoto": False,
            "watermarkVideo": False,
            "watermarkText": "",
            "liveServer": "",
            "liveServerKey": ""
        }
        if prof.get("username") and str(prof.get("username")) != "" and prof.get("watermarkText") == "":
            prof.set("watermarkText", "OnlyFans.com/{}".format(prof.get("username")))
        return prof



    # returns list of settings and their classes
    # ["settingVariableName","pageProfile","inputType-text"]
    
def get_settings_variables():
    return [

        ### Profile ###

        ["coverImage","profile","file"],
        ["profilePhoto","profile","file"],
        ["username","profile","text"],
        ["displayName","profile","text"],
        ["subscriptionPrice","profile","text"],
        ["referralReward","dropdown"],
        ["about","profile","text"],
        ["location","profile","text"],
        ["websiteURL","profile","text"],

        #### Account / Advanced ###

        # id="input-email"
        ["email","advanced","text"],
        # id="old_password_input"
        ["password","advanced","text"],
        # id="new_password_input"
        ["newPassword","advanced","text"],
        # id="new_password2_input"
        ["confirmPassword","advanced","checkbox"],

        ### Chats / Messages ###
        # welcome message toggle
        # welcome message text
        # welcome message file
        # welcome message record voice, video
        # welcome message price
        # welcome message submit

        # hide outgoing message toggle
        # show full text of message in the notification email

        ### Notifications ###

        # id="push-notifications"
        ["emailNotifs","notifications","toggle"],
        # id="email-notifications"
        ["emailNotifsReferral","notifications","checkbox"],
        ["emailNotifsStream","notifications","toggle"],
        ["emailNotifsSubscriber","notifications","toggle"],
        ["emailNotifsTip","notifications","toggle"],
        ["emailNotifsRenewal","notifications","toggle"],
        # this is a dropdown
        ["emailNotifsLikes","notifications","dropdown"],
        ["emailNotifsPosts","notifications","toggle"],
        ["emailNotifsPrivMessages","notifications","toggle"],
        ["siteNotifs","notifications","toggle"],
        ["siteNotifsComment","notifications","toggle"],
        ["siteNotifsFavorite","notifications","toggle"],
        ["siteNotifsDiscounts","notifications","toggle"],
        ["siteNotifsSubscriber","notifications","toggle"],
        ["siteNotifsTip","notifications","toggle"],
        ["toastNotifsComment","notifications","toggle"],
        ["toastNotifsFavorite","notifications","toggle"],
        ["toastNotifsSubscriber","notifications","toggle"],
        ["toastNotifsTip","notifications","toggle"],

        ### Security ###

        ["fullyPrivate","security","checkbox"],
        ["enableComments","security","toggle"],
        ["showFansCount","security","toggle"],
        ["showPostsTip","security","toggle"],
        ["publicFriendsList","security","toggle"],
        ["ipCountry","security","list"],
        # id="input-blocked-ips"
        ["ipIP","security","list"],
        # id="hasWatermarkPhoto"
        ["watermarkPhoto","security","toggle"],
        # id="hasWatermarkVideo"
        ["watermarkVideo","security","toggle"],
        # placeholder="Watermark custom text"
        ["watermarkText","security","text"],

        ### Story ###
        # allow message replies - nobody
        # allow message replies - subscribers

        ### Other ###

        ["liveServer","other","text"],
        ["liveServerKey","other","text"],
        ["welcomeMessageToggle","other","toggle"],
        ["welcomeMessageText","other","text"],

    ]




#!/usr/bin/python3
# 3/28/2019 Skeetzo
# OnlySnarf.py menu system

### doesn't work:
# upload & backup (requires upload via local added to main script)
# settings menu -> "Incorrect Index"
import time
import random
import os
import shutil
import datetime
import json
import sys
import pathlib
import pkg_resources
from PyInquirer import prompt
##
from OnlySnarf.src.colorize import colorize
# from OnlySnarf.src.cron import Cron
from OnlySnarf.src.classes import Discount, Promotion
from OnlySnarf.src.message import Message
from OnlySnarf.src import google as Google
# from OnlySnarf.src.promotion import Promotion
from OnlySnarf.src.settings import Settings
from OnlySnarf.src.snarf import Snarf

###################
##### Globals #####
###################

ASCII = "\n ________         .__          _________                     _____ \n \
\\_____  \\   ____ |  | ___.__./   _____/ ____ _____ ________/ ____\\\n \
 /   |   \\ /    \\|  |<   |  |\\_____  \\ /    \\\\__  \\\\_   _ \\   __\\ \n \
/    |    \\   |  \\  |_\\___  |/        \\   |  \\/ __ \\ |  |\\/| |   \n \
\\_______  /___|  /____/ ____/_______  /___|  (____  \\\\__|  |_|   \n \
        \\/     \\/     \\/            \\/     \\/     \\/              \n"
snarf = Snarf()
VERSION = str(pkg_resources.get_distribution("onlysnarf").version)

#####################
##### Functions #####
#####################

# Action

def ask_action():
    menu_prompt = {
        'type': 'list',
        'name': 'action',
        'message': 'Please select an action:',
        'choices': ['Back', 'Discount', 'Message', 'Post', 
            # 'Promotion'
        ]
    }
    if str(Settings.is_debug()) == "True":
        menu_prompt["choices"].append("Promotion")
    answers = prompt(menu_prompt)
    return answers['action']

def action_menu():
    action = ask_action()
    if (action == 'Back'): main()
    elif (action == 'Discount'): discount_menu()
    elif (action == 'Message'): message_menu()
    elif (action == 'Post'): post_menu()
    elif (action == 'Promotion'): promotion_menu()
    else: main()

# Discount

def discount_menu():
    if not Settings.is_debug():
        print("### Not Available ###")
        return
    discount = Discount()
    discount.apply()
    main()

# Message

def message_menu():
    message = Message()
    message.send()
    main()

# Post

def post_menu():
    message = Message()
    message.post()
    main()

# Profile

def profile_menu():
    print("### Not Available ###")
    # Profile.menu(snarf)
    main()

# Promotion

def promotion_menu():
    if not Settings.is_debug():
        print("### Not Available ###")
        return
    promotion = Promotion()
    # add menu in promotion that asks for which kind
    # promotion.create_trial_link()
    promotion.apply_to_user()
    main()

# Settings

def settings_menu():
    Settings.menu()
    main()
    
# Main

def header():
    if not Settings.is_debug(): os.system('clear')
    print(colorize(ASCII, 'header'))
    print(colorize('version {}\n'.format(VERSION), 'green'))
    user_header()
    settings_header()


def settings_header():
    Settings.header()

def user_header():
    print("User:")
    print(" - Username = {}".format(Settings.get_username()))
    pass_ = ""
    if str(Settings.get_password()) != "":
        pass_ = "******"
    print(" - Password = {}".format(pass_))
    print('\r')

def menu():
    menu_prompt = {
        'type': 'list',
        'name': 'menu',
        'message': 'Please select an option:',
        'choices': ['Action', 'Profile', 'Settings', 'Exit']
    }
    answers = prompt(menu_prompt)
    return answers['menu']

def main_menu():
    action = menu()
    if (action == 'Action'): action_menu()
    elif (action == 'Profile'): profile_menu()
    elif (action == 'Settings'): settings_menu()
    else: exit()

def main():
    time.sleep(1)
    try:
        header()
        settings_header()
        main_menu()
    except Exception as e:
        Settings.dev_print(e)

#################################################################################################

import atexit
def exit_handler():
    print("Shnarrf?")
    exit()
atexit.register(exit_handler)

import signal
def signal_handler(sig, frame):
    print('Shnnnarf?')
    exit()
signal.signal(signal.SIGINT, signal_handler)
  
def exit():
    sys.exit(0)

######################################################

if __name__ == "__main__":
    try:
        main()
    except Exception as e:
        print("Shhhhhnnnnnarf!")
    finally:
        exit()


#!/usr/bin/python3
# main OnlySnarf class

import random
import os
import shutil
import datetime
import json
import sys
import pathlib
import time
##
from . import cron as Cron
from .classes import Discount, Promotion
from .driver import Driver
from .settings import Settings
from .user import User
from .profile import Profile

#####################
##### OnlySnarf #####
#####################

class Snarf:

    def __init__(self):
        pass
        # self.profile = Profile({})

    ####################
    ##### Discount #####
    ####################

    @staticmethod
    def discount(discount=None):
        if not discount: discount = Settings.get_discount()
        try: discount.apply()
        except Exception as e: Settings.dev_print(e)
        Snarf.exit()

    ################
    ##### Exit #####
    ################

    @staticmethod
    def exit():
        if Settings.is_show_window():
            Settings.maybe_print("Skipping: Window Close")
            return
        Driver.exit()

    ###################
    ##### Message #####
    ###################

    @staticmethod
    def message(message=None):
        if not message: message = Settings.get_message()
        try: message.send()
        except Exception as e: Settings.dev_print(e)
        Snarf.exit()
                
    ################
    ##### Post #####
    ################

    @staticmethod
    def post(message=None):
        if not message: message = Settings.get_message()
        try: message.post()
        except Exception as e: Settings.dev_print(e)
        Snarf.exit()

    ###################
    ##### Profile #####
    ###################

    @staticmethod
    def profile(profile=None):
        if not profile: profile = Settings.get_profile()
        try: profile.update()
        except Exception as e: Settings.dev_print(e)
        Snarf.exit()
        
    #####################
    ##### Promotion #####
    #####################

    # def give_trial(user):
    #     print("Applying Promotion: "+user)
    #     link = Driver.get_new_trial_link()
    #     text = "Here's your free trial link!\n"+link
    #     Settings.dev_print("Link: "+str(text))
    #     # Settings.send_email(email, text)

    @staticmethod
    def promotion(promotion=None):
        if not promotion: promotion = Settings.get_promotion()
        try: promotion.apply_to_user()
        except Exception as e: Settings.dev_print(e)
        Snarf.exit()

    #################
    ##### Reset #####
    #################

    @staticmethod
    def reset():
        Driver.reset()

    #################
    ##### Users #####
    #################

    @staticmethod
    def get_following():
        users = []
        try: users = User.get_following()
        except Exception as e: Settings.dev_print(e)
        return users

    @staticmethod
    def get_users():
        users = []
        try: users = User.get_all_users()
        except Exception as e: Settings.dev_print(e)
        return users

    ###############
    ##### Dev #####
    ###############

    @staticmethod
    def test():
        print('0/3 : Deleting Locals')
        print('1/3 : Testing')
        print('TESTING: Users')
        response = Driver.users_get()
        # return True
        print('TESTING: Following')
        response = Driver.following_get()
        # return True
        print('TESTING: Settings - Get')
        response = Driver.settings_get_all()
        return True
        print('TESTING: Cron')
        response = Cron.test()
        if not response or response == None:
            print("Error: Failed to test crons")
        reset_ = reset()
        if not reset_:
            return print("Error: Failed to Reset")
        return True

################################################################################################################################################

def main():
    # try:
    from .file import File
    File.remove_local()
    Settings.set_prompt(False)
    Settings.set_confirm(False)
    action = Settings.get_action()
    print("Running - {}".format(action))
    ## Actions
    success = False
    if str(action) == "test":
        success = Snarf.test()
    elif str(action) == "post":
        success = Snarf.post()
    elif str(action) == "message":
        success = Snarf.message()
    elif str(action) == "discount":
        success = Snarf.discount()
    elif str(action) == "promotion":
        success = Snarf.promotion()
    elif str(action) == "profile":
        success = Snarf.profile()
    else:
        print("Warning: Missing Method")
    Snarf.exit()
    # except Exception as e:
    #     Settings.dev_print(e)
    #     print("Shnarf!")
    # finally:
    #     sys.exit(0)

################################################################################################################################################

if __name__ == "__main__":
    main()